<div class="omnilib">
    <img src="_static/omnilib.png" />
    <h4>The Omnilib Project</h4>
    <span>
        <a href="https://omnilib.dev">Omnilib</a> is a group of MIT licensed software libraries developed under a
        common, inclusive <a href="https://omnilib.dev/code_of_conduct.html">Code of Conduct</a>.
        We are committed to providing a welcoming and open space for all contributors who adhere to these rules.
    </span>
</div>
<div class="omnilib-badges">
    <a href="https://twitter.com/omnilib" alt="Omnilib on Twitter">
        <i class="fab fa-twitter"></i>
    </a>
    &nbsp;
    <a href="https://github.com/omnilib" alt="Omnilib on Github">
        <i class="fab fa-github"></i>
    </a>
    &nbsp;
    <a href="https://github.com/{{theme_github_user}}/{{theme_github_repo}}" alt="View site source">
        <i class="fas fa-code"></i>
    </a>

</div>

<link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.15.4/css/all.css" integrity="sha384-DyZ88mC6Up2uqS4h/KRgHuoeGwBcD4Ng9SiP4dIRy0EXTlnuz47vAwmeGwVChigm" crossorigin="anonymous">
<link rel="stylesheet" href="https://use.typekit.net/ale5jph.css">